import argparse

import retro

parser = argparse.ArgumentParser()
parser.add_argument(
    "--game",
    default="Airstriker-Genesis",
    help="the name or path for the game to run",
)
parser.add_argument(
    "--state",
    help="the initial state file to load, minus the extension",
)
parser.add_argument(
    "--scenario",
    "-s",
    default="scenario",
    help="the scenario file to load, minus the extension",
)
parser.add_argument("--record", "-r", action="store_true", help="record bk2 movies")
parser.add_argument(
    "--verbose",
    "-v",
    action="count",
    default=1,
    help="increase verbosity (can be specified multiple times)",
)
parser.add_argument(
    "--quiet",
    "-q",
    action="count",
    default=0,
    help="decrease verbosity (can be specified multiple times)",
)
parser.add_argument(
    "--obs-type",
    "-o",
    default="image",
    choices=["image", "ram"],
    help="the observation type, either `image` (default) or `ram`",
)
parser.add_argument(
    "--players",
    "-p",
    type=int,
    default=1,
    help="number of players/agents (default: 1)",
)
args = parser.parse_args()

obs_type = (
    retro.Observations.IMAGE if args.obs_type == "image" else retro.Observations.RAM
)
env = retro.make(
    args.game,
    args.state or retro.State.DEFAULT,
    scenario=args.scenario,
    record=args.record,
    players=args.players,
    obs_type=obs_type,
)
verbosity = args.verbose - args.quiet
try:
    while True:
        ob = env.reset()
        t = 0
        totrew = [0] * args.players
        while True:
            ac = env.action_space.sample()
            ob, rew, terminated, truncated, info = env.step(ac)
            t += 1
            if t % 10 == 0:
                if verbosity > 1:
                    infostr = ""
                    if info:
                        infostr = ", info: " + ", ".join(
                            ["%s=%i" % (k, v) for k, v in info.items()],
                        )
                    print(("t=%i" % t) + infostr)
                env.render()
            if args.players == 1:
                rew = [rew]
            for i, r in enumerate(rew):
                totrew[i] += r
                if verbosity > 0:
                    if r > 0:
                        print(
                            "t=%i p=%i got reward: %g, current reward: %g"
                            % (t, i, r, totrew[i]),
                        )
                    if r < 0:
                        print(
                            "t=%i p=%i got penalty: %g, current reward: %g"
                            % (t, i, r, totrew[i]),
                        )
            if terminated or truncated:
                env.render()
                try:
                    if verbosity >= 0:
                        if args.players > 1:
                            print(
                                "done! total reward: time=%i, reward=%r" % (t, totrew),
                            )
                        else:
                            print(
                                "done! total reward: time=%i, reward=%d"
                                % (t, totrew[0]),
                            )
                        input("press enter to continue")
                        print()
                    else:
                        input("")
                except EOFError:
                    exit(0)
                break
except KeyboardInterrupt:
    exit(0)
